html {
    width: 100%;
}
body {
    min-width: 600px;
    max-width: 900px;
    width: 75%;
    margin: auto;
}
.content {
    margin: 2em;
}
h1 {
    margin: 0 0 0 0;
    font-family: 'Julius Sans One', sans-serif;
    font-size: 1.8em;
    text-transform: uppercase;
}

.header {
    border-radius: 10px;
    background-color: darkgray;
    padding: 1%;
}

.header img {
  float: left;
  width: 100px;
  height: 100px;
  background: #555;
}

.header h1 {
    position: left;
    top: 18px;
    left: 10px;
    color: #2e484c;
    font-family: 'Julius Sans One', sans-serif;
    text-transform: uppercase;
    font-size: 1.8em;
    display: inline;
    vertical-align: top;
}

p.title {
    margin: 2em 0 0 0;
    color: #2e484c;
    font-family: 'Julius Sans One', sans-serif;
    font-size: 1.8em;
    text-transform: uppercase;
}
a:link {
    font-weight: bold;
    text-decoration: none;
    color: #0d8ba1;
}
a:visited {
    font-weight: bold;
    text-decoration: none;
    color: #1a5952;
}
a:hover, a:focus, a:active {
    text-decoration: underline;
    color: #9685BA;
}
p {
    font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
    font-size: 1.2em;
    text-align: justify;
    text-justify: inter-word;
}

footer{
    display: table;
    text-align: center;
    margin-left: auto;
    margin-right: auto;
}